@@ -1,5 +1,8 @@
 #![allow(clippy::new_without_default)]
 
+#[cfg(not(any(feature = "bat-printer", feature = "syntect-printer")))]
+compile_error!("Either feature \"bat-printer\" or \"syntect-printer\" must be enabled");
+
 #[cfg(feature = "bat-printer")]
 pub mod bat;
 pub mod chunk;
