@@ -20,12 +20,12 @@ use hgrep::bat::BatPrinter;
 use hgrep::syntect::SyntectPrinter;
 
 fn cli<'a>() -> App<'a> {
-    #[cfg(feature = "bat-printer")]
-    const DEFAULT_PRINTER: &str = "bat";
-
-    #[cfg(all(not(feature = "bat-printer"), feature = "syntect-printer"))]
+    #[cfg(feature = "syntect-printer")]
     const DEFAULT_PRINTER: &str = "syntect";
 
+    #[cfg(all(not(feature = "syntect-printer"), feature = "bat-printer"))]
+    const DEFAULT_PRINTER: &str = "bat";
+
     let app = App::new("hgrep")
         .version(env!("CARGO_PKG_VERSION"))
         .about(
@@ -40,7 +40,7 @@ fn cli<'a>() -> App<'a> {
                 .long("min-context")
                 .takes_value(true)
                 .value_name("NUM")
-                .default_value("5")
+                .default_value("3")
                 .about("Minimum lines of leading and trailing context surrounding each match"),
         )
         .arg(
@@ -49,7 +49,7 @@ fn cli<'a>() -> App<'a> {
                 .long("max-context")
                 .takes_value(true)
                 .value_name("NUM")
-                .default_value("10")
+                .default_value("6")
                 .about("Maximum lines of leading and trailing context surrounding each match"),
         )
         .arg(
@@ -81,7 +81,7 @@ fn cli<'a>() -> App<'a> {
         .arg(
             Arg::new("list-themes")
                 .long("list-themes")
-                .about("List all theme names available for --theme option"),
+                .about("List all available theme names and their samples. Samples show the output where 'let' is searched. The names can be used at --theme option"),
         )
         .arg(
             Arg::new("printer")
@@ -142,7 +142,7 @@ fn cli<'a>() -> App<'a> {
         )
         .arg(
             Arg::new("ascii-lines").long("ascii-lines").about(
-                "Use ascii characters for drawing border lines instead of Unicode characters",
+                "Use ASCII characters for drawing border lines instead of Unicode characters",
             ),
         );
 
@@ -255,7 +255,7 @@ fn cli<'a>() -> App<'a> {
                 Arg::new("line-regexp")
                     .short('x')
                     .long("line-regexp")
-                    .about("Only show matches surrounded by line boundaries. This is equivalent to putting ^...$ around all of the search patterns"),
+                    .about("Only show matches surrounded by line boundaries. This is equivalent to putting ^...$ around the search pattern"),
             )
             .arg(
                 Arg::new("pcre2")
@@ -355,24 +355,6 @@ fn app() -> Result<bool> {
         ),
     };
 
-    if matches.is_present("list-themes") {
-        #[cfg(feature = "syntect-printer")]
-        if printer_kind == PrinterKind::Syntect {
-            hgrep::syntect::list_themes(io::stdout().lock())?;
-            return Ok(true);
-        }
-
-        #[cfg(feature = "bat-printer")]
-        if printer_kind == PrinterKind::Bat {
-            for theme in BatPrinter::new(PrinterOptions::default()).themes() {
-                println!("{}", theme);
-            }
-            return Ok(true);
-        }
-
-        unreachable!();
-    }
-
     let min_context = matches
         .value_of("min-context")
         .unwrap()
@@ -392,18 +374,27 @@ fn app() -> Result<bool> {
             .context("could not parse \"tab\" option value as unsigned integer")?;
     }
 
+    #[cfg(feature = "bat-printer")]
     let theme_env = env::var("BAT_THEME").ok();
-    if let Some(var) = &theme_env {
-        printer_opts.theme = Some(var);
+    #[cfg(feature = "bat-printer")]
+    if printer_kind == PrinterKind::Bat {
+        if let Some(var) = &theme_env {
+            printer_opts.theme = Some(var);
+        }
     }
     if let Some(theme) = matches.value_of("theme") {
         printer_opts.theme = Some(theme);
     }
 
     let is_grid = matches.is_present("grid");
-    if let Ok("plain" | "header" | "numbers") = env::var("BAT_STYLE").as_ref().map(String::as_str) {
-        if !is_grid {
-            printer_opts.grid = false;
+    #[cfg(feature = "bat-printer")]
+    if printer_kind == PrinterKind::Bat {
+        if let Ok("plain" | "header" | "numbers") =
+            env::var("BAT_STYLE").as_ref().map(String::as_str)
+        {
+            if !is_grid {
+                printer_opts.grid = false;
+            }
         }
     }
     if matches.is_present("no-grid") && !is_grid {
@@ -465,6 +456,22 @@ fn app() -> Result<bool> {
         }
     }
 
+    if matches.is_present("list-themes") {
+        #[cfg(feature = "syntect-printer")]
+        if printer_kind == PrinterKind::Syntect {
+            hgrep::syntect::list_themes(io::stdout().lock(), &printer_opts)?;
+            return Ok(true);
+        }
+
+        #[cfg(feature = "bat-printer")]
+        if printer_kind == PrinterKind::Bat {
+            BatPrinter::new(printer_opts).list_themes()?;
+            return Ok(true);
+        }
+
+        unreachable!();
+    }
+
     #[cfg(feature = "ripgrep")]
     if let Some(pattern) = matches.value_of("PATTERN") {
         let paths = matches.values_of_os("PATH");
