@@ -8,14 +8,14 @@ use grep_regex::{RegexMatcher, RegexMatcherBuilder};
 use grep_searcher::{BinaryDetection, MmapChoice, Searcher, SearcherBuilder, Sink, SinkMatch};
 use ignore::overrides::OverrideBuilder;
 use ignore::types::{Types, TypesBuilder};
-use ignore::{WalkBuilder, WalkParallel, WalkState};
+use ignore::{Walk, WalkBuilder};
+use rayon::iter::ParallelBridge;
 use rayon::prelude::*;
 use std::env;
 use std::ffi::OsStr;
 use std::fs::File;
 use std::io;
 use std::path::PathBuf;
-use std::sync::mpsc::channel;
 use std::sync::Mutex;
 
 // Note: 'main is a lifetime of scope of main() function
@@ -178,7 +178,7 @@ impl<'main> Config<'main> {
         self
     }
 
-    fn build_walker(&self, mut paths: impl Iterator<Item = &'main OsStr>) -> Result<WalkParallel> {
+    fn build_walker(&self, mut paths: impl Iterator<Item = &'main OsStr>) -> Result<Walk> {
         let target = paths.next().unwrap();
 
         let mut builder = OverrideBuilder::new(target);
@@ -212,7 +212,7 @@ impl<'main> Config<'main> {
             builder.add_custom_ignore_filename(".rgignore");
         }
 
-        Ok(builder.build_parallel())
+        Ok(builder.build())
     }
 
     fn build_regex_matcher(&self, pat: &str) -> Result<RegexMatcher> {
@@ -328,17 +328,24 @@ pub fn grep<'main, P: Printer + Sync>(
     paths: Option<impl Iterator<Item = &'main OsStr>>,
     config: Config<'main>,
 ) -> Result<bool> {
-    let paths = if let Some(paths) = paths {
-        walk(paths, &config)?
+    let entries = if let Some(paths) = paths {
+        config.build_walker(paths)?
     } else {
         let cwd = env::current_dir()?;
         let paths = std::iter::once(cwd.as_os_str());
-        walk(paths, &config)?
+        config.build_walker(paths)?
     };
 
-    if paths.is_empty() {
-        return Ok(false);
-    }
+    let paths = entries.filter_map(|entry| match entry {
+        Ok(entry) => {
+            if entry.file_type().map(|t| t.is_file()).unwrap_or(false) {
+                Some(Ok(entry.into_path()))
+            } else {
+                None
+            }
+        }
+        Err(err) => Some(Err(anyhow::Error::new(err))),
+    });
 
     if config.pcre2 {
         Ripgrep::with_pcre2(pat, config, printer)?.grep(paths)
@@ -347,32 +354,62 @@ pub fn grep<'main, P: Printer + Sync>(
     }
 }
 
-fn walk<'main>(
-    paths: impl Iterator<Item = &'main OsStr>,
-    config: &Config<'main>,
-) -> Result<Vec<PathBuf>> {
-    let walker = config.build_walker(paths)?;
+#[derive(Default)]
+struct LineRegions<'a> {
+    ranges: &'a [(usize, usize)],
+    offset: usize,
+}
 
-    let (tx, rx) = channel();
-    walker.run(|| {
-        // This function is called per threads for initialization.
-        let tx = tx.clone();
-        Box::new(move |entry| match entry {
-            Ok(entry) => {
-                if entry.file_type().map(|f| f.is_file()).unwrap_or(false) {
-                    tx.send(Ok(entry.into_path())).unwrap();
-                }
-                WalkState::Continue
-            }
-            Err(err) => {
-                tx.send(Err(anyhow::Error::new(err))).unwrap();
-                WalkState::Quit
-            }
-        })
-    });
-    drop(tx); // Notify sender finishes
+impl<'a> LineRegions<'a> {
+    fn new(ranges: &'a [(usize, usize)]) -> Self {
+        Self { ranges, offset: 0 }
+    }
 
-    rx.into_iter().collect()
+    fn line_ranges(&mut self, line_len: usize) -> Vec<(usize, usize)> {
+        // Invariant: self.ranges is sorted and not over-wrapped
+        let line_start = self.offset;
+        let line_end = line_start + line_len;
+
+        let mut ret = vec![];
+        let mut next_start_idx = 0;
+        for (idx, (range_start, range_end)) in self.ranges.iter().copied().enumerate() {
+            // ls < le < rs < re
+            if line_end < range_start {
+                break;
+            }
+
+            let start = if range_start < line_start {
+                0
+            } else if line_start <= range_start && range_start < line_end {
+                range_start - line_start
+            } else {
+                // line_end <= range_start
+                break;
+            };
+
+            let end = if range_end < line_start {
+                // This range is not useful for later lines
+                next_start_idx = idx;
+                continue;
+            } else if line_start <= range_end && range_end < line_end {
+                range_end - line_start
+            } else {
+                // line_end <= range_end
+                line_end - line_start
+            };
+
+            if start < end {
+                ret.push((start, end));
+            }
+        }
+
+        if next_start_idx > 0 {
+            self.ranges = &self.ranges[next_start_idx..];
+        }
+        self.offset = line_end; // Offset for next line
+
+        ret
+    }
 }
 
 struct Matches<'a, M: Matcher> {
@@ -398,54 +435,22 @@ impl<'a, M: Matcher> Sink for Matches<'a, M> {
         let line_number = mat.line_number().unwrap();
         let path = &self.path;
 
-        let range = self
-            .matcher
-            .find(mat.bytes())
-            .map_err(|e| io::Error::new(io::ErrorKind::Other, format!("{}", e)))?
-            .map(|m| (m.start(), m.end()));
+        let mut ranges = vec![];
+        self.matcher
+            .find_iter(mat.bytes(), |m| {
+                ranges.push((m.start(), m.end()));
+                true
+            })
+            .map_err(|e| io::Error::new(io::ErrorKind::Other, format!("{}", e)))?;
+        let mut regions = LineRegions::new(&ranges);
 
-        #[inline]
-        fn region_in_line(
-            range: Option<(usize, usize)>,
-            line_start: usize,
-            line_end: usize,
-        ) -> Option<(usize, usize)> {
-            let (start, end) = range?;
-
-            let region_start = if start < line_start {
-                0
-            } else if line_start <= start && start < line_end {
-                start - line_start
-            } else {
-                // line_end <= start
-                return None;
-            };
-
-            let region_end = if end < line_start {
-                return None;
-            } else if line_start <= end && end < line_end {
-                end - line_start
-            } else {
-                // line_end <= end
-                line_end - line_start
-            };
-
-            if region_start == region_end {
-                return None;
-            }
-            Some((region_start, region_end))
-        }
-
-        let mut start = 0;
         let mut line_number = line_number;
         for line in mat.lines() {
-            let end = start + line.len();
             self.buf.push(GrepMatch {
                 path: path.to_owned(),
                 line_number,
-                range: region_in_line(range, start, end),
+                ranges: regions.line_ranges(line.len()),
             });
-            start = end;
             line_number += 1;
         }
 
@@ -486,12 +491,15 @@ where
         }
     }
 
-    fn search(&self, path: PathBuf) -> Result<Vec<GrepMatch>> {
+    // Return Result<Option<Vec<_>>> instead of Result<Vec<_>> to make the `filter_map` predicate easy
+    // in `grep()` method
+    fn search(&self, path: PathBuf) -> Result<Option<Vec<GrepMatch>>> {
         if let Some(count) = &self.count {
             if *count.lock().unwrap() == 0 {
-                return Ok(vec![]);
+                return Ok(None);
             }
         }
+
         let file = File::open(&path)?;
         let mut searcher = self.config.build_searcher();
         let mut matches = Matches {
@@ -500,12 +508,16 @@ where
             matcher: &self.matcher,
             buf: vec![],
         };
+
         searcher.search_file(&self.matcher, &file, &mut matches)?;
-        Ok(matches.buf)
+        if matches.buf.is_empty() {
+            return Ok(None);
+        }
+
+        Ok(Some(matches.buf))
     }
 
-    fn grep_file(&self, path: PathBuf) -> Result<bool> {
-        let matches = self.search(path)?;
+    fn print_matches(&self, matches: Vec<GrepMatch>) -> Result<bool> {
         let (min, max) = (self.config.min_context, self.config.max_context);
         let mut found = false;
         for file in Files::new(matches.into_iter().map(Ok), min, max) {
@@ -515,10 +527,17 @@ where
         Ok(found)
     }
 
-    fn grep(&self, paths: Vec<PathBuf>) -> Result<bool> {
+    fn grep<I>(&self, paths: I) -> Result<bool>
+    where
+        I: Iterator<Item = Result<PathBuf>> + Send,
+    {
         paths
-            .into_par_iter()
-            .map(|path| self.grep_file(path))
+            .par_bridge()
+            .filter_map(|path| match path {
+                Ok(path) => self.search(path).transpose(),
+                Err(err) => Some(Err(err)),
+            })
+            .map(|matches| self.print_matches(matches?))
             .try_reduce(|| false, |a, b| Ok(a || b))
     }
 }
@@ -552,7 +571,7 @@ mod tests {
                 let lines: Vec<_> = file.contents.split_inclusive(|b| *b == b'\n').collect();
                 for lmat in file.line_matches.iter_mut() {
                     // Reset `lmat.range` to None since ranges in `expected` are `None`
-                    let (start, end) = mem::take(&mut lmat.range).unwrap();
+                    let (start, end) = mem::take(&mut lmat.ranges)[0];
                     let line = lines[lmat.line_number as usize - 1];
                     let matched_part = &line[start..end];
                     assert_eq!(
@@ -727,7 +746,7 @@ mod tests {
                 let end = s.next().unwrap().parse().unwrap();
                 line_matches.push(LineMatch {
                     line_number,
-                    range: Some((start, end)),
+                    ranges: vec![(start, end)],
                 })
             }
         }
@@ -792,4 +811,90 @@ mod tests {
             c.pcre2(true);
         });
     }
+
+    macro_rules! line_regions_tests {
+        {$(
+            $name:ident(
+                $ranges:expr,
+                $line_lens:expr,
+                [$($expected:expr),*],
+            );
+        )+} => {
+            $(
+                #[test]
+                fn $name() {
+                    let ranges = &$ranges;
+                    let line_lens = &$line_lens;
+                    let expected = &[
+                        $(
+                            &$expected[..],
+                        )*
+                    ];
+
+                    let mut r = LineRegions::new(ranges);
+                    for (idx, len) in line_lens.iter().copied().enumerate() {
+                        assert_eq!(&r.line_ranges(len), expected[idx], "index={}", idx);
+                    }
+                }
+            )+
+        }
+    }
+
+    line_regions_tests! {
+        region_no_region(
+            [],
+            [2, 2, 2],
+            [[], [], []],
+        );
+        region_entire_line(
+            [(1, 5)],
+            [2, 2, 2],
+            [[(1, 2)], [(0, 2)], [(0, 1)]],
+        );
+        region_entire_lines(
+            [(2, 6)],
+            [2, 2, 2, 2],
+            [[], [(0, 2)], [(0, 2)], []],
+        );
+        region_entire_region(
+            [(1, 5)],
+            [10],
+            [[(1, 5)]],
+        );
+        region_left_most(
+            [(0, 3)],
+            [5],
+            [[(0, 3)]],
+        );
+        region_right_most(
+            [(3, 5)],
+            [5],
+            [[(3, 5)]],
+        );
+        region_accross_lines(
+            [(3, 5)],
+            [2, 2, 2, 2],
+            [[], [(1, 2)], [(0, 1)], []],
+        );
+        regions_accross_lines(
+            [(3, 5), (9, 11)],
+            [2, 2, 2, 2, 2, 2, 2],
+            [[], [(1, 2)], [(0, 1)], [], [(1, 2)], [(0, 1)], []],
+        );
+        regions_entire_lines(
+            [(1, 2), (5, 6)],
+            [4, 4],
+            [[(1, 2)], [(1, 2)]],
+        );
+        regions_multi_regions_in_line(
+            [(1, 2), (3, 4)],
+            [10],
+            [[(1, 2), (3, 4)]],
+        );
+        regions_multi_regions_in_lines(
+            [(1, 2), (3, 4), (6, 7), (8, 9)],
+            [5, 5],
+            [[(1, 2), (3, 4)], [(1, 2), (3, 4)]],
+        );
+    }
 }
