@@ -1,12 +1,10 @@
-use crate::chunk::File;
-use crate::chunk::Line;
+use crate::chunk::{File, Line};
 use crate::printer::{Printer, PrinterOptions, TermColorSupport, TextWrapMode};
 use anyhow::Result;
 use flate2::read::ZlibDecoder;
 use memchr::{memchr_iter, Memchr};
 use rgb2ansi256::rgb_to_ansi256;
 use std::cmp;
-use std::collections::HashSet;
 use std::ffi::OsStr;
 use std::fmt;
 use std::io::Write;
@@ -27,6 +25,14 @@ use unicode_width::{UnicodeWidthChar, UnicodeWidthStr};
 const SYNTAX_SET_BIN: &[u8] = include_bytes!("../assets/syntaxes.bin");
 const THEME_SET_BIN: &[u8] = include_bytes!("../assets/themes.bin");
 
+fn load_bat_themes() -> Result<ThemeSet> {
+    Ok(bincode::deserialize_from(ZlibDecoder::new(THEME_SET_BIN))?)
+}
+
+fn load_syntax_set() -> Result<SyntaxSet> {
+    Ok(bincode::deserialize_from(ZlibDecoder::new(SYNTAX_SET_BIN))?)
+}
+
 pub trait LockableWrite<'a> {
     type Locked: Write;
     fn lock(&'a self) -> Self::Locked;
@@ -39,17 +45,42 @@ impl<'a> LockableWrite<'a> for Stdout {
     }
 }
 
-pub fn list_themes<W: Write>(mut out: W) -> Result<()> {
-    let mut seen = HashSet::new();
-    let bat_defaults = bincode::deserialize_from(ZlibDecoder::new(THEME_SET_BIN))?;
-    let defaults = ThemeSet::load_defaults();
-    for themes in &[bat_defaults, defaults] {
-        for name in themes.themes.keys() {
-            if !seen.contains(name) {
-                writeln!(out, "{}", name)?;
-                seen.insert(name);
-            }
+pub fn list_themes<W: Write>(out: W, opts: &PrinterOptions<'_>) -> Result<()> {
+    let syntaxes = load_syntax_set()?;
+    list_themes_with_syntaxes(out, opts, &syntaxes)
+}
+
+fn list_themes_with_syntaxes<W: Write>(
+    mut out: W,
+    opts: &PrinterOptions<'_>,
+    syntaxes: &SyntaxSet,
+) -> Result<()> {
+    let mut themes = vec![];
+    themes.extend(load_bat_themes()?.themes.into_iter());
+    themes.extend(ThemeSet::load_defaults().themes.into_iter());
+    themes.sort_by(|l, r| l.0.cmp(&r.0));
+
+    let syntax = syntaxes.find_syntax_by_name("Rust").unwrap();
+    let sample_file = File::sample_file();
+
+    let mut last = None;
+    for (name, theme) in themes.iter() {
+        if Some(name) == last {
+            continue; // Remove duplicates
         }
+
+        let mut drawer = Drawer::new(&mut out, opts, theme, &sample_file.chunks);
+        drawer.canvas.set_bold()?;
+        write!(drawer.canvas, "{:?}", name)?;
+        drawer.canvas.draw_newline()?;
+        drawer.canvas.draw_sample()?;
+        writeln!(drawer.canvas)?;
+
+        let hl = LineHighlighter::new(syntax, theme, syntaxes);
+        drawer.draw_file(&sample_file, hl)?;
+        writeln!(drawer.canvas)?;
+
+        last = Some(name);
     }
     Ok(())
 }
@@ -85,6 +116,7 @@ impl fmt::Display for PrintError {
     }
 }
 
+#[derive(Debug)]
 struct Token<'line> {
     style: Style,
     text: &'line str,
@@ -196,31 +228,171 @@ impl<'a, 'line: 'a> DrawEvents<'a, 'line> {
     }
 }
 
-struct Canvas<'file, W: Write> {
+#[inline]
+#[allow(clippy::many_single_char_names)]
+fn blend_fg_color(fg: Color, bg: Color) -> Color {
+    if fg.a == 0xff || fg.a == 0 || fg.a == 1 {
+        return fg; // 0 and 1 are special cases for 16 colors and 256 colors themes
+    }
+    let x = fg.a as u32;
+    let y = (255 - fg.a) as u32;
+    let r = (fg.r as u32 * x + bg.r as u32 * y) / 255;
+    let g = (fg.g as u32 * x + bg.g as u32 * y) / 255;
+    let b = (fg.b as u32 * x + bg.b as u32 * y) / 255;
+    Color {
+        r: r as u8,
+        g: g as u8,
+        b: b as u8,
+        a: 255,
+    }
+}
+
+#[inline]
+fn color_average(c: Color) -> u8 {
+    ((c.r as u32 + c.g as u32 + c.b as u32) / 3) as u8
+}
+
+#[inline]
+fn weak_blend_fg_color(mut fg: Color, bg: Color) -> Color {
+    let fg_avg = color_average(fg);
+    let bg_avg = color_average(bg);
+    let heuristic_ratio = if fg_avg > bg_avg {
+        if fg_avg - bg_avg >= 200 {
+            4 // Vivid dark theme uses further weaker gutter foreground
+        } else {
+            3 // Dark theme uses weaker gutter foreground than light theme
+        }
+    } else {
+        2
+    };
+    fg.a /= heuristic_ratio;
+    blend_fg_color(fg, bg)
+}
+
+#[derive(Debug)]
+struct Palette {
+    foreground: Color,
+    background: Color,
+    match_bg: Color,
+    match_lnum_fg: Color,
+    region_fg: Color,
+    region_bg: Color,
+    gutter_fg: Color,
+}
+
+impl Palette {
+    const NO_COLOR: Color = Color {
+        r: 0,
+        g: 0,
+        b: 0,
+        a: 1, // Special color which means pass though
+    };
+    const YELLOW_COLOR_16: Color = Color {
+        r: 3, // Yellow
+        g: 0,
+        b: 0,
+        a: 0,
+    };
+    const BLACK_COLOR_16: Color = Color {
+        r: 0, // Black
+        g: 0,
+        b: 0,
+        a: 0,
+    };
+    const ANSI16: Palette = Palette {
+        foreground: Self::NO_COLOR,
+        background: Self::NO_COLOR,
+        match_bg: Self::NO_COLOR,
+        match_lnum_fg: Self::YELLOW_COLOR_16,
+        region_fg: Self::BLACK_COLOR_16,
+        region_bg: Self::YELLOW_COLOR_16,
+        gutter_fg: Self::NO_COLOR,
+    };
+
+    fn new(theme: &Theme) -> Self {
+        let background = theme.settings.background.unwrap_or(Self::NO_COLOR);
+        let foreground = theme.settings.foreground.unwrap_or(Self::NO_COLOR);
+        let foreground = blend_fg_color(foreground, background);
+
+        if foreground.a == 1 && background.a == 1 {
+            return Self::ANSI16;
+        }
+
+        // gutter and gutter_foreground are not fit to show line numbers and borders in some color themes
+        let gutter_fg = weak_blend_fg_color(foreground, background);
+        let match_lnum_fg = blend_fg_color(foreground, background);
+
+        let match_bg = theme
+            .settings
+            .line_highlight
+            .unwrap_or_else(|| weak_blend_fg_color(foreground, background));
+
+        let (region_fg, region_bg) = if let Some(bg) = theme.settings.find_highlight {
+            let fg = theme
+                .settings
+                .find_highlight_foreground
+                .unwrap_or(foreground);
+            let fg = blend_fg_color(fg, bg);
+            (fg, bg)
+        } else {
+            (background, foreground)
+        };
+
+        Self {
+            foreground,
+            background,
+            match_bg,
+            match_lnum_fg,
+            region_fg,
+            region_bg,
+            gutter_fg,
+        }
+    }
+
+    fn is_ansi16(&self) -> bool {
+        self.foreground.a == 1 && self.foreground.r <= 7
+    }
+}
+
+struct Canvas<W: Write> {
     out: W,
-    theme: &'file Theme,
     true_color: bool,
     has_background: bool,
-    match_bg: Option<Color>,
-    region_fg: Option<Color>,
-    region_bg: Option<Color>,
+    palette: Palette,
     current_fg: Option<Color>,
     current_bg: Option<Color>,
 }
 
-impl<'file, W: Write> Deref for Canvas<'file, W> {
+impl<W: Write> Deref for Canvas<W> {
     type Target = W;
     fn deref(&self) -> &Self::Target {
         &self.out
     }
 }
-impl<'file, W: Write> DerefMut for Canvas<'file, W> {
+impl<W: Write> DerefMut for Canvas<W> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         &mut self.out
     }
 }
 
-impl<'file, W: Write> Canvas<'file, W> {
+impl<W: Write> Canvas<W> {
+    fn new(out: W, opts: &PrinterOptions<'_>, theme: &Theme) -> Self {
+        let palette = if opts.color_support == TermColorSupport::Ansi16 {
+            Palette::ANSI16
+        } else {
+            Palette::new(theme)
+        };
+
+        Self {
+            out,
+            true_color: opts.color_support == TermColorSupport::True,
+            has_background: !palette.is_ansi16() && opts.background_color,
+            palette,
+            current_fg: None,
+            current_bg: None,
+        }
+    }
+
     fn draw_spaces(&mut self, num: usize) -> Result<()> {
         for _ in 0..num {
             self.out.write_all(b" ")?;
@@ -241,16 +413,14 @@ impl<'file, W: Write> Canvas<'file, W> {
         match c.a {
             0 if c.r <= 7 => write!(self.out, "\x1b[{}m", c.r + code)?, // 16 colors; e.g. 3 => 33 (Yellow), 6 => 36 (Cyan) (code=30)
             0 => write!(self.out, "\x1b[{};5;{}m", code + 8, c.r)?, // 256 colors; code=38 for fg, code=48 for bg
-            1 => { /* Pass through. Do nothing */ }
+            1 => write!(self.out, "\x1b[0m")?, // Pass though. Reset color to set default terminal font color
             _ if self.true_color => {
-                write!(self.out, "\x1b[{};2;{};{};{}m", code + 8, c.r, c.g, c.b)?
+                write!(self.out, "\x1b[{};2;{};{};{}m", code + 8, c.r, c.g, c.b)?;
+            }
+            _ => {
+                let c = rgb_to_ansi256(c.r, c.g, c.b);
+                write!(self.out, "\x1b[{};5;{}m", code + 8, c,)?;
             }
-            _ => write!(
-                self.out,
-                "\x1b[{};5;{}m",
-                code + 8,
-                rgb_to_ansi256(c.r, c.g, c.b),
-            )?,
         }
         Ok(())
     }
@@ -273,18 +443,13 @@ impl<'file, W: Write> Canvas<'file, W> {
 
     fn set_default_bg(&mut self) -> Result<()> {
         if self.has_background {
-            if let Some(bg) = self.theme.settings.background {
-                self.set_bg(bg)?;
-            }
+            self.set_bg(self.palette.background)?;
         }
         Ok(())
     }
 
     fn set_default_fg(&mut self) -> Result<()> {
-        if let Some(fg) = self.theme.settings.foreground {
-            self.set_fg(fg)?;
-        }
-        Ok(())
+        self.set_fg(self.palette.foreground)
     }
 
     fn set_background(&mut self, c: Color) -> Result<()> {
@@ -304,6 +469,16 @@ impl<'file, W: Write> Canvas<'file, W> {
         Ok(())
     }
 
+    fn unset_bold(&mut self) -> Result<()> {
+        self.out.write_all(b"\x1b[22m")?;
+        Ok(())
+    }
+
+    fn unset_underline(&mut self) -> Result<()> {
+        self.out.write_all(b"\x1b[24m")?;
+        Ok(())
+    }
+
     fn set_font_style(&mut self, style: FontStyle) -> Result<()> {
         if style.contains(FontStyle::BOLD) {
             self.set_bold()?;
@@ -316,10 +491,10 @@ impl<'file, W: Write> Canvas<'file, W> {
 
     fn unset_font_style(&mut self, style: FontStyle) -> Result<()> {
         if style.contains(FontStyle::BOLD) {
-            self.out.write_all(b"\x1b[22m")?;
+            self.unset_bold()?;
         }
         if style.contains(FontStyle::UNDERLINE) {
-            self.out.write_all(b"\x1b[24m")?;
+            self.unset_underline()?;
         }
         Ok(())
     }
@@ -332,26 +507,28 @@ impl<'file, W: Write> Canvas<'file, W> {
     }
 
     fn set_match_bg_color(&mut self) -> Result<()> {
-        if let Some(bg) = self.match_bg {
-            self.set_bg(bg)?;
-        }
-        Ok(())
+        self.set_bg(self.palette.match_bg)
     }
 
     fn set_match_style(&mut self, style: Style) -> Result<()> {
-        self.set_fg(style.foreground)?;
         self.set_match_bg_color()?;
+        self.set_fg(style.foreground)?;
         self.set_font_style(style.font_style)
     }
 
     fn set_region_color(&mut self) -> Result<()> {
-        if let Some(c) = self.region_fg {
-            self.set_fg(c)?;
-        }
-        if let Some(c) = self.region_bg {
-            self.set_bg(c)?;
-        }
-        Ok(())
+        self.set_fg(self.palette.region_fg)?;
+        self.set_bg(self.palette.region_bg)
+    }
+
+    fn set_gutter_color(&mut self) -> Result<()> {
+        self.set_fg(self.palette.gutter_fg)?;
+        self.set_default_bg()
+    }
+
+    fn set_match_lnum_color(&mut self) -> Result<()> {
+        self.set_fg(self.palette.match_lnum_fg)?;
+        self.set_default_bg()
     }
 
     fn fill_spaces(&mut self, written_width: usize, max_width: usize) -> Result<()> {
@@ -360,6 +537,26 @@ impl<'file, W: Write> Canvas<'file, W> {
         }
         Ok(())
     }
+
+    fn draw_sample_row(&mut self, colors: &[(&str, Color)]) -> Result<()> {
+        for (name, color) in colors {
+            write!(self.out, "    {} ", name)?;
+            self.set_bg(*color)?;
+            self.out.write_all(b"    \x1b[0m")?;
+        }
+        writeln!(self.out)?;
+        self.current_fg = None;
+        self.current_bg = None;
+        Ok(())
+    }
+
+    #[rustfmt::skip]
+    fn draw_sample(&mut self) -> Result<()> {
+        self.draw_sample_row(&[("Foreground:   ", self.palette.foreground), ("Background:   ", self.palette.background)])?;
+        self.draw_sample_row(&[("MatchLineBG:  ", self.palette.match_bg),   ("MatchLineNum: ", self.palette.match_lnum_fg)])?;
+        self.draw_sample_row(&[("MatchRegionFG:", self.palette.region_fg),  ("MatchRegionBG:", self.palette.region_bg)])?;
+        self.draw_sample_row(&[("GutterFG:     ", self.palette.gutter_fg)])
+    }
 }
 
 struct LineChars<'a> {
@@ -418,7 +615,10 @@ impl<'a> LineHighlighter<'a> {
     fn highlight<'line>(&mut self, line: &'line str) -> Vec<Token<'line>> {
         let ops = self.parse_state.parse_line(line, self.syntaxes);
         HighlightIterator::new(&mut self.hl_state, &ops, line, &self.hl)
-            .map(|(style, text)| Token { style, text })
+            .map(|(mut style, text)| {
+                style.foreground = blend_fg_color(style.foreground, style.background);
+                Token { style, text }
+            })
             .collect()
     }
 }
@@ -465,52 +665,21 @@ struct Drawer<'file, W: Write> {
     grid: bool,
     term_width: u16,
     lnum_width: u16,
-    background: bool,
     first_only: bool,
-    gutter_color: Color,
     wrap: bool,
     tab_width: u16,
     chars: LineChars<'file>,
-    canvas: Canvas<'file, W>,
+    canvas: Canvas<W>,
 }
 
 impl<'file, W: Write> Drawer<'file, W> {
-    fn new(out: W, opts: &PrinterOptions, theme: &'file Theme, chunks: &[(u64, u64)]) -> Self {
+    fn new(out: W, opts: &PrinterOptions<'_>, theme: &'file Theme, chunks: &[(u64, u64)]) -> Self {
         let last_lnum = chunks.last().map(|(_, e)| *e).unwrap_or(0);
         let mut lnum_width = num_digits(last_lnum);
         if chunks.len() > 1 {
             lnum_width = cmp::max(lnum_width, 3); // Consider '...' in gutter
         }
 
-        let gutter_color = theme
-            .settings
-            .gutter_foreground
-            .or(theme.settings.minimap_border)
-            .unwrap_or(Color {
-                r: 128,
-                g: 128,
-                b: 128,
-                a: 255,
-            });
-
-        let (region_fg, region_bg) = if let Some(bg) = theme.settings.find_highlight {
-            (theme.settings.find_highlight_foreground, Some(bg))
-        } else {
-            (None, theme.settings.selection)
-        };
-
-        let canvas = Canvas {
-            theme,
-            true_color: opts.color_support == TermColorSupport::True,
-            has_background: opts.background_color,
-            region_fg,
-            region_bg,
-            current_fg: None,
-            current_bg: None,
-            match_bg: theme.settings.line_highlight.or(theme.settings.background),
-            out,
-        };
-
         let chars = if opts.ascii_lines {
             ASCII_LINE_CHARS
         } else {
@@ -521,13 +690,11 @@ impl<'file, W: Write> Drawer<'file, W> {
             grid: opts.grid,
             term_width: opts.term_width,
             lnum_width,
-            background: opts.background_color,
-            gutter_color,
             wrap: opts.text_wrap == TextWrapMode::Char,
             tab_width: opts.tab_width as u16,
             first_only: opts.first_only,
             chars,
-            canvas,
+            canvas: Canvas::new(out, opts, theme),
         }
     }
 
@@ -541,8 +708,7 @@ impl<'file, W: Write> Drawer<'file, W> {
     }
 
     fn draw_horizontal_line(&mut self, sep: &str) -> Result<()> {
-        self.canvas.set_fg(self.gutter_color)?;
-        self.canvas.set_default_bg()?;
+        self.canvas.set_gutter_color()?;
         let gutter_width = self.gutter_width();
         for _ in 0..gutter_width - 2 {
             self.canvas.write_all(self.chars.horizontal.as_bytes())?;
@@ -556,29 +722,27 @@ impl<'file, W: Write> Drawer<'file, W> {
 
     fn draw_line_number(&mut self, lnum: u64, matched: bool) -> Result<()> {
         if matched {
-            self.canvas.set_default_fg()?;
+            self.canvas.set_match_lnum_color()?;
         } else {
-            self.canvas.set_fg(self.gutter_color)?;
+            self.canvas.set_gutter_color()?;
         }
-        self.canvas.set_default_bg()?;
         let width = num_digits(lnum);
         self.canvas
             .draw_spaces((self.lnum_width - width) as usize)?;
         write!(self.canvas, " {}", lnum)?;
         if self.grid {
             if matched {
-                self.canvas.set_fg(self.gutter_color)?;
+                self.canvas.set_gutter_color()?;
             }
             write!(self.canvas, " {}", self.chars.vertical)?;
         }
         self.canvas.set_default_bg()?;
-        write!(self.canvas, " ")?;
+        self.canvas.write_all(b" ")?;
         Ok(()) // Do not reset color because another color text will follow
     }
 
     fn draw_wrapping_gutter(&mut self) -> Result<()> {
-        self.canvas.set_fg(self.gutter_color)?;
-        self.canvas.set_default_bg()?;
+        self.canvas.set_gutter_color()?;
         self.canvas.draw_spaces(self.lnum_width as usize + 2)?;
         if self.grid {
             write!(self.canvas, "{} ", self.chars.vertical)?;
@@ -587,8 +751,7 @@ impl<'file, W: Write> Drawer<'file, W> {
     }
 
     fn draw_separator_line(&mut self) -> Result<()> {
-        self.canvas.set_fg(self.gutter_color)?;
-        self.canvas.set_default_bg()?;
+        self.canvas.set_gutter_color()?;
         // + 1 for left margin and - 3 for length of "..."
         let left_margin = self.lnum_width + 1 - 3;
         self.canvas.draw_spaces(left_margin as usize)?;
@@ -770,7 +933,7 @@ impl<'file, W: Write> Drawer<'file, W> {
         self.canvas.set_default_fg()?;
         self.canvas.set_bold()?;
         write!(self.canvas, " {}", path)?;
-        if self.background {
+        if self.canvas.has_background {
             self.canvas
                 .fill_spaces(path.width_cjk() + 1, self.term_width as usize)?;
         }
@@ -787,10 +950,16 @@ impl<'file, W: Write> Drawer<'file, W> {
         }
         Ok(())
     }
+
+    fn draw_file(&mut self, file: &File, hl: LineHighlighter) -> Result<()> {
+        self.draw_header(&file.path)?;
+        self.draw_body(file, hl)?;
+        self.draw_footer()
+    }
 }
 
 fn load_themes(name: Option<&str>) -> Result<ThemeSet> {
-    let bat_defaults: ThemeSet = bincode::deserialize_from(ZlibDecoder::new(THEME_SET_BIN))?;
+    let bat_defaults: ThemeSet = load_bat_themes()?;
     match name {
         None => Ok(bat_defaults),
         Some(name) if bat_defaults.themes.contains_key(name) => Ok(bat_defaults),
@@ -813,7 +982,7 @@ pub struct SyntectAssets {
 
 impl SyntectAssets {
     pub fn load(theme: Option<&str>) -> Result<Self> {
-        let syntax_set = bincode::deserialize_from(ZlibDecoder::new(SYNTAX_SET_BIN))?;
+        let syntax_set = load_syntax_set()?;
         let theme_set = load_themes(theme)?;
         Ok(Self {
             syntax_set,
@@ -857,7 +1026,7 @@ where
     pub fn new(writer: W, opts: PrinterOptions<'main>) -> Result<Self> {
         Ok(Self {
             writer,
-            syntaxes: bincode::deserialize_from(ZlibDecoder::new(SYNTAX_SET_BIN))?,
+            syntaxes: load_syntax_set()?,
             themes: load_themes(opts.theme)?,
             opts,
         })
@@ -918,11 +1087,8 @@ where
         let theme = self.theme();
         let syntax = self.find_syntax(&file.path)?;
 
-        let mut drawer = Drawer::new(&mut buf, &self.opts, theme, &file.chunks);
-        drawer.draw_header(&file.path)?;
         let hl = LineHighlighter::new(syntax, theme, &self.syntaxes);
-        drawer.draw_body(&file, hl)?;
-        drawer.draw_footer()?;
+        Drawer::new(&mut buf, &self.opts, theme, &file.chunks).draw_file(&file, hl)?;
 
         // Take lock here to print files in serial from multiple threads
         let mut output = self.writer.lock();
@@ -941,6 +1107,7 @@ mod tests {
     use std::fs;
     use std::mem;
     use std::path::PathBuf;
+    use std::str;
 
     lazy_static! {
         static ref ASSETS: SyntectAssets = SyntectAssets::load(None).unwrap();
@@ -992,10 +1159,7 @@ mod tests {
                         base += end;
                     }
                     if ranges.len() > 0 {
-                        lmats.push(LineMatch {
-                            line_number: lnum,
-                            ranges,
-                        });
+                        lmats.push(LineMatch::new(lnum, ranges));
                     }
                 } else {
                     break;
@@ -1098,7 +1262,7 @@ mod tests {
             run_uitest(file, outfile, f);
         }
 
-        macro_rules! uitest {
+        macro_rules! uitests {
             ($($input:ident($f:expr),)+) => {
                 $(
                     #[test]
@@ -1109,7 +1273,7 @@ mod tests {
             }
         }
 
-        uitest!(
+        uitests!(
             test_default(|_| {}),
             test_background(|o| {
                 o.background_color = true;
@@ -1243,6 +1407,60 @@ mod tests {
         );
     }
 
+    // Separate module from `ui` since pretty_assertions is too slow for showing diff between byte sequences.
+    mod list_themes {
+        use super::*;
+
+        fn run_parametrized_uitest_list_themes(
+            mut input: &str,
+            f: fn(&mut PrinterOptions<'_>) -> (),
+        ) {
+            if input.starts_with("test_") {
+                input = &input["test_".len()..];
+            }
+            let file = format!("list_themes_{}.out", input);
+            let expected = Path::new("testdata").join("syntect").join(file);
+            let expected = fs::read(&expected).unwrap();
+
+            let mut opts = PrinterOptions::default();
+            opts.term_width = 80;
+            opts.color_support = TermColorSupport::True;
+            f(&mut opts);
+
+            let mut got = vec![];
+            list_themes_with_syntaxes(&mut got, &opts, &ASSETS.syntax_set).unwrap();
+
+            assert_eq!(
+                expected,
+                got,
+                "expected:\n{}\ngot:\n{}",
+                str::from_utf8(&expected).unwrap(),
+                str::from_utf8(&got).unwrap()
+            );
+        }
+
+        macro_rules! list_theme_uitests {
+            ($($input:ident($f:expr),)+) => {
+                $(
+                    #[test]
+                    fn $input() {
+                        run_parametrized_uitest_list_themes(stringify!($input), $f);
+                    }
+                )+
+            }
+        }
+
+        list_theme_uitests! {
+            test_default(|_| {}),
+            test_no_grid(|o| {
+                o.grid = false;
+            }),
+            test_background(|o| {
+                o.background_color = true;
+            }),
+        }
+    }
+
     #[derive(Debug)]
     struct DummyError;
     impl std::error::Error for DummyError {}
@@ -1300,19 +1518,6 @@ mod tests {
         assert!(msg.contains("Unknown theme"), "message={:?}", msg);
     }
 
-    #[test]
-    fn test_list_themes() {
-        let mut buf = vec![];
-        list_themes(&mut buf).unwrap();
-        let out = String::from_utf8(buf).unwrap();
-
-        // From bat's assets
-        assert!(out.contains("Monokai Extended\n"), "output={:?}", out);
-
-        // From default assets
-        assert!(out.contains("base16-ocean.dark\n"), "output={:?}", out);
-    }
-
     #[test]
     fn test_print_nothing() {
         let file = File::new(PathBuf::from("x.txt"), vec![], vec![], vec![]);
@@ -1369,7 +1574,7 @@ mod tests {
         assert!(
             found,
             "line={:?}",
-            std::str::from_utf8(this_is_test_line).unwrap()
+            str::from_utf8(this_is_test_line).unwrap()
         );
     }
 }
